==> mwalk-0.2.2.tgz: mwalk-0.2.2/src/mwalk/visitor/BasicMetricVisitor.java <==
/* 
     File : BasicMetricVisitor.java
   Author : Robert Chalmers

 Original : December 1999
  Revised : 

  Content : A basic implementation of a visitor that gathers metric
            data and saves it out to a file. 
             
*/

package mwalk.visitor;


import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import mwalk.core.Tree;
import mwalk.core.TreeNode;
import mwalk.core.Config;
import mwalk.util.MetricCount;
import mwalk.util.VisitException;


/**
 * A basic implementation of a visitor that gathers metric
 * data and saves it out to a file.
 *
 * @author Robert Chalmers
 * @version 1.0 
 */
public class BasicMetricVisitor extends MetricVisitor {

    /** Base of keys in configuration file */
    private static final String CFG_BASE = "BasicMetricVisitor.";
    /** Default file name if none supplied */
    private static final String DEF_FILENAME = "tree-metric.basic";

    protected boolean optRun = false;
    protected boolean optMulti = false;


    /**
     * Default constructor.
     */
    public BasicMetricVisitor() {

	super( DEF_FILENAME );
    }

    /**
     * Constructor.
     *
     * @param <code>String</code> new default filename
     */
    public BasicMetricVisitor( String filename ) {

	super( filename );
    }


    /**
     * Constructor.
     *
     * @param <code>long</code> number of periods to cover session
     */
    public BasicMetricVisitor( long periods ) {

	super( DEF_FILENAME, periods );
    }

    /**
     * Constructor.
     *
     * @param <code>String</code> new default filename
     * @param <code>long</code> number of periods to cover session
     */
    public BasicMetricVisitor( String filename, long periods ) {

	super( filename, periods );
    }


    /**
     * Retrieve the current filename.
     *
     * @return <code>String</code> current filename
     */    
    public String getFilename() {

	return( filename + ((optMulti) ? ".mopt" : "") );
    }


    /**
     * Print visitor's header info at start of file.
     *
     * @param <code>Tree</code> current tree instance
     */
    protected void printHeader( Tree tree ) {

	super.printHeader( tree );

	ps.println( "# simulation start:" + tree.getStart() + " stop:" + tree.getStop() + " periods:" + periods );
	ps.println( "# multicast optimized:" + optMulti );
	ps.println( "#" );
	ps.println( "# Time\tRecvs\tUlinks\tMlinks\tMetric\tM est.\tLu\tLm/Lu\tR^k" );
    }


    /**
     * Initialize the visitor state.
     *
     * @param <code>Tree</code> current tree instance
     */
    public boolean init( Tree tree ) {

	// check for mcast optimization from config file
	optRun = cfgBoolean( CFG_BASE + "optimize" );

	return( super.init( tree ) );
    }

    /**
     * Check whether the visitor wants another pass over the tree.
     *
     * @param <code>Tree</code> vistited tree 
     * @return <code>boolean</code> whether to walk again
     */
    public boolean again( Tree tree ) {

	// when again stops for non-optMulti, reset and run for optMulti if desired
	if( ! super.again( tree ) ) {
	    if( ! optMulti && optRun  ) {
		optMulti = true;
		init( tree );
	    } else
		return( false );
	}

	return( true );
    }


    protected MetricCount doVisit( Tree tree, TreeNode node, Vector list ) {

	MetricCount count = new MetricCount();
	int klinks = 0;
	
	// handle counts from each child link
	for( Enumeration kids = list.elements(); kids.hasMoreElements(); ) 
	    try {
		MetricCount kcount = (MetricCount)kids.nextElement();
		if( kcount.receivers > 0 ) {
		    Config.verbose( this, "Got a count from child: " + kcount.toString() );
		    count.add( kcount );
		    klinks++;
		} else
		    Config.verbose( this, "Got an empty count from child." );
		
	    } catch( NoSuchElementException nse ) {}
	
	// add ourselves into the pot
	return( addLinks( count, klinks ) );
    }


    protected MetricCount addLinks( MetricCount count, int mlinks ) {

	// add ourselves into the pot (with optional optimization for multicast)
	if( count.receivers > 0 ) {
	    Config.verbose( this, "Adding counts for " + count.receivers + " receivers." );
	    count.ucastLinks += count.receivers;
	    count.mcastLinks += (optMulti) ? 1 : mlinks;
	}

	return( count );
    }

	
    protected void process( Tree tree ) throws VisitException {

	try {
	    // compute the final metric data from the counts	    
	    MetricCount count = (MetricCount)tree.getSource().data;
	    double metric = 0, mest = 0, aucast = 0, cratio = 0, cpower = 0;
	    if( count.receivers > 0 ) {
		metric = 1.0 - (double)count.mcastLinks / (double)count.ucastLinks;
		mest = 1.0 - Math.pow( count.receivers, -0.2 );
		aucast = (double)count.ucastLinks / (double)count.receivers;
		cratio = (double)count.mcastLinks / aucast;
		cpower = Math.pow( count.receivers, 0.8 );
	    }
		
	    // build a buffer with each field separated by tabs
	    StringBuffer buf = new StringBuffer( 128 );
	    char tab = '\t';
	    buf.append( time ).append( tab ).append( count.toString() ).append( tab );
	    buf.append( metric ).append( tab ).append( mest ).append( tab );
	    buf.append( aucast ).append( tab ).append( cratio ).append( tab ).append( cpower );
	    
	    // convert buffer to one string
	    String line = buf.toString();
	    Config.verbose( this, line );
	    printData( tree, line );

	} catch( Exception e ) {
	    throw new VisitException( "error printing metric data at time " + time, e );
	}
    }
}

==> mwalk-0.3.0.tgz: mwalk-0.3.0/src/mwalk/visitor/BasicMetricVisitor.java <==
/* 
     File : BasicMetricVisitor.java
   Author : Robert Chalmers

 Original : December 1999
  Revised : 

  Content : A basic implementation of a visitor that gathers metric
            data and saves it out to a file. 
             
*/

package mwalk.visitor;


import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import mwalk.core.Tree;
import mwalk.core.TreeNode;
import mwalk.core.Config;
import mwalk.util.MetricCount;
import mwalk.util.VisitException;


/**
 * A basic implementation of a visitor that gathers metric
 * data and saves it out to a file.
 *
 * @author Robert Chalmers
 * @version 1.0 
 */
public class BasicMetricVisitor extends MetricVisitor {

    /** Base of keys in configuration file */
    private static final String CFG_BASE = "BasicMetricVisitor.";
    /** Default file name if none supplied */
    private static final String DEF_FILENAME = "tree-metric.basic";

    protected boolean optRun = false;
    protected boolean optMulti = false;


    /**
     * Default constructor.
     */
    public BasicMetricVisitor() {

	super( DEF_FILENAME );
    }

    /**
     * Constructor.
     *
     * @param <code>String</code> new default filename
     */
    public BasicMetricVisitor( String filename ) {

	super( filename );
    }


    /**
     * Constructor.
     *
     * @param <code>long</code> number of periods to cover session
     */
    public BasicMetricVisitor( long periods ) {

	super( DEF_FILENAME, periods );
    }

    /**
     * Constructor.
     *
     * @param <code>String</code> new default filename
     * @param <code>long</code> number of periods to cover session
     */
    public BasicMetricVisitor( String filename, long periods ) {

	super( filename, periods );
    }


    /**
     * Retrieve the current filename.
     *
     * @return <code>String</code> current filename
     */    
    public String getFilename() {

	return( filename + ((optMulti) ? ".mopt" : "") );
    }


    /**
     * Print visitor's header info at start of file.
     *
     * @param <code>Tree</code> current tree instance
     */
    protected void printHeader( Tree tree ) {

	super.printHeader( tree );

	ps.println( "# simulation start:" + tree.getStart() + " stop:" + tree.getStop() + " periods:" + periods );
	ps.println( "# multicast optimized:" + optMulti );
	ps.println( "#" );
	ps.println( "# Time\tRecvs\tUlinks\tMlinks\tMetric\tM est.\tLu\tLm/Lu\tR^k" );
    }


    /**
     * Initialize the visitor state.
     *
     * @param <code>Tree</code> current tree instance
     */
    public boolean init( Tree tree ) {

	// check for mcast optimization from config file
	optRun = cfgBoolean( CFG_BASE + "optimize" );

	return( super.init( tree ) );
    }

    /**
     * Check whether the visitor wants another pass over the tree.
     *
     * @param <code>Tree</code> vistited tree 
     * @return <code>boolean</code> whether to walk again
     */
    public boolean again( Tree tree ) {

	// when again stops for non-optMulti, reset and run for optMulti if desired
	if( ! super.again( tree ) ) {
	    if( ! optMulti && optRun  ) {
		optMulti = true;
		init( tree );
	    } else
		return( false );
	}

	return( true );
    }


    protected MetricCount doVisit( Tree tree, TreeNode node, Vector list ) {

	MetricCount count = new MetricCount();
	int klinks = 0;
	
	// handle counts from each child link
	for( Enumeration kids = list.elements(); kids.hasMoreElements(); ) 
	    try {
		MetricCount kcount = (MetricCount)kids.nextElement();
		if( kcount.receivers > 0 ) {
		    Config.verbose( this, "Got a count from child: " + kcount.toString() );
		    count.add( kcount );
		    klinks++;
		} else
		    Config.verbose( this, "Got an empty count from child." );
		
	    } catch( NoSuchElementException nse ) {}
	
	// add ourselves into the pot
	return( addLinks( count, klinks ) );
    }


    protected MetricCount addLinks( MetricCount count, int mlinks ) {

	// add ourselves into the pot (with optional optimization for multicast)
	if( count.receivers > 0 ) {
	    Config.verbose( this, "Adding counts for " + count.receivers + " receivers." );
	    count.ucastLinks += count.receivers;
	    count.mcastLinks += (optMulti) ? 1 : mlinks;
	}

	return( count );
    }

	
    protected void process( Tree tree ) throws VisitException {

	try {
	    // compute the final metric data from the counts	    
	    MetricCount count = (MetricCount)tree.getSource().data.get( "metric" );
	    double metric = 0, mest = 0, aucast = 0, cratio = 0, cpower = 0;
	    if( count.receivers > 0 ) {
		metric = 1.0 - (double)count.mcastLinks / (double)count.ucastLinks;
		mest = 1.0 - Math.pow( count.receivers, -0.2 );
		aucast = (double)count.ucastLinks / (double)count.receivers;
		cratio = (double)count.mcastLinks / aucast;
		cpower = Math.pow( count.receivers, 0.8 );
	    }
		
	    // build a buffer with each field separated by tabs
	    StringBuffer buf = new StringBuffer( 128 );
	    char tab = '\t';
	    buf.append( time ).append( tab ).append( count.toString() ).append( tab );
	    buf.append( metric ).append( tab ).append( mest ).append( tab );
	    buf.append( aucast ).append( tab ).append( cratio ).append( tab ).append( cpower );
	    
	    // convert buffer to one string
	    String line = buf.toString();
	    Config.verbose( this, line );
	    printData( tree, line );

	} catch( Exception e ) {
	    throw new VisitException( "error printing metric data at time " + time, e );
	}
    }
}
